%MACRO PersonScore(filesave, respfile, thetafile, itemfile, formuse, personID,
				   nSubjects, mcmcseed, nburnin, ntune, nmcmc, nthin); 

*******************************************************************************
******			CONVERTING RESPONSE DATA TO GLOBAL TEST DATA		 	 ******
*******************************************************************************;

* Import person responses;
DATA &respfile.; SET filesave.&respfile.; RUN;
PROC SORT DATA=&respfile.; BY &personID.; RUN;

* Create empty data with all possible item responses;
DATA empty; 
	RETAIN &personID. ppvt3a_1-ppvt3a_204 ppvtrm_1-ppvtrm_175 ppvt3b_1-ppvt3b_204;
	&personID. = .;
	ARRAY aRm(175) ppvtrm_1-ppvtrm_175;
		DO i=1 TO 175; aRm(i) = .; END;
	ARRAY a3a(204) ppvt3a_1-ppvt3a_204;
	ARRAY a3b(204) ppvt3b_1-ppvt3b_204;
		DO i=1 TO 204; a3a(i) = .; a3b(i) = .; END; RUN;	

* Merge real data with empty data;
DATA responses; MERGE empty &respfile.; 
	KEEP &personID. ppvt3a_1--ppvt3b_204 internalID;
	internalID = _N_; 
	RUN;

* Import item key data for 459 items;
DATA &itemfile.; SET filesave.&itemfile.; RUN;

* Stacking into each test and merging in item info;
DATA ppvt3a; SET responses;
	KEEP &personID. ppvt3a_1--ppvt3a_204 internalID; RUN;
DATA ppvt3a; SET ppvt3a; 
	ARRAY appvt3a(204) ppvt3a_1--ppvt3a_204;
		DO i=1 TO 204;
			Item3a=i; ppvt3a=appvt3a(i); OUTPUT;
		END; KEEP &personID. Item3a ppvt3a internalID; RUN; 
PROC SORT DATA=ppvt3a; BY Item3a &personID.; RUN;
PROC SORT DATA=&itemfile.; BY Item3a; RUN;
DATA ppvt3a; MERGE ppvt3a &itemfile.; BY Item3a; 
	WHERE NMISS(Item3a)=0; RUN;
DATA ppvtRm; SET responses;
	KEEP &personID. ppvtRm_1--ppvtRm_175 internalID; RUN;
DATA ppvtRm; SET ppvtRm;
	ARRAY appvtrm(175) ppvtrm_1--ppvtrm_175;
		DO i=1 TO 175;
			ItemRm=i; ppvtRm=appvtrm(i); OUTPUT;
		END; KEEP &personID. ItemRm ppvtRm internalID; RUN;
PROC SORT DATA=ppvtRm; BY ItemRm &personID.; RUN;
PROC SORT DATA=&itemfile.; BY ItemRm; RUN;
DATA ppvtRm; MERGE ppvtRm &itemfile.; BY ItemRm;
	WHERE NMISS(ItemRm)=0; RUN;
DATA ppvt3b; SET responses;
	KEEP &personID. ppvt3b_1--ppvt3b_204 internalID; RUN;
DATA ppvt3b; SET ppvt3b;
	ARRAY appvt3b(204) ppvt3b_1--ppvt3b_204;
		DO i=1 TO 204;
			Item3b=i; ppvt3b=appvt3b(i); OUTPUT;
		END; KEEP &personID. Item3b ppvt3b internalID; RUN; 
PROC SORT DATA=ppvt3b; BY Item3b &personID.; RUN;
PROC SORT DATA=&itemfile.; BY Item3b; RUN;
DATA ppvt3b; MERGE ppvt3b &itemfile.; BY Item3b;
	WHERE NMISS(Item3b)=0; RUN;

* Merge stacked files together;
PROC SORT DATA=&itemfile.; BY testnum; RUN;
PROC SORT DATA=ppvt3a; BY testnum &personID.; RUN;
PROC SORT DATA=ppvtRm; BY testnum &personID.; RUN;
PROC SORT DATA=ppvt3b; BY testnum &personID.; RUN;
DATA allstack; MERGE ppvt3a ppvtRm ppvt3b; BY testnum; RUN; 
DATA teststack; 
	RETAIN &personID. item testnum Item3a ItemRm Item3b ppvt3a ppvtRm ppvt3b internalID;
	SET allstack;
	* Moving responses to one column;
	IF N(ppvt3a, ppvtRm, ppvt3b) GE 2 THEN resp=PPVT&formuse.;
	ELSE IF N(ppvt3a)=1 THEN resp=ppvt3a;
	ELSE IF N(ppvtRm)=1 THEN resp=ppvtRm;
	ELSE IF N(ppvt3b)=1 THEN resp=ppvt3b;
	* Checking for 3s;
	IF resp=3 THEN resp=0;
	LABEL 	resp=   "resp: Item Response to be Used"
			ppvt3a=	"ppvt3a: PPVT-3a Item Response"
			ppvtRm=	"ppvtRm: PPVT-Rm Item Response"
			ppvt3b=	"ppvt3b: PPVT-3b Item Response";
RUN;

* Transposing to multivariate;
PROC SORT DATA=teststack; BY &personID. testnum; RUN;
PROC TRANSPOSE DATA=teststack OUT=testmult PREFIX=resp;
	BY &personID.; ID testnum; VAR resp; RUN;
DATA testmult; SET testmult; DROP _NAME_ _LABEL_; RUN;

* Deleting unnecessary files;
PROC DATASETS LIB=WORK NOLIST; 
	DELETE allstack empty ppvt3a ppvt3b ppvtRm; 
RUN; QUIT;


*******************************************************************************
******			SCORING PERSONS USING ITEM PARAMETERS AND RESPONSES		 ******
*******************************************************************************;

* Files available:
&itemfile. = original item-level data of item parameters
&respfile. = original person-level item response data
testmult = person-level global test item responses (&personID. resp1-resp45)
testmult = person by item item characteristics and responses (dv = resp);

*** Scoring magic goes here, end up with data called "work.thetas";
%MACRO mcmc_scorePPVT(dataname, orderedID, respname, aname, bname, nsubjects, mcmcseed, 
                      nburnin, ntune, nmcmc, nthin);
*Macro variables:
*	dataname  = name of SAS data set containing stacked data;
*	orderedid = name of variable in data that is an ordered id for counter loop (internalID from macro);
*	respname  = name of variable in data that contains dichotomous item responses (missing ok);
*	aname     = name of variable in data that contains calibrated a parameters;
*	bname     = name of variable in data that contains calibrated b parameters;
*   nsubjects = highest value for orderedID;
*   mcmcseed  = seed value for MCMC initialization - usually 0;
*  	nburnin   = number of burnin iterations for MCMC chain - suggest 1000;
*   ntune     = number of tuning iterations for PROC MCMC - suggest 500;
*   nmcmc     = number of MCMC iterations, excluding burnin - suggest 2000;
*   nthin     = thinning interval for MCMC iterations - suggest 10;

PROC SORT DATA=&dataname.; BY &orderedid.; RUN; *order data;

%LET I = 1;
DATA thetas; RUN; * temporary data set where results will be dumped;
ODS LISTING CLOSE;
%DO I = 1 %TO &nsubjects.;
	DATA tempmcmc; SET &dataname.; IF &orderedID. = &I; RUN; *extract ONLY data for case I;

	PROC MCMC DATA=tempmcmc SEED=&mcmcseed. NTU=&ntune. NBI=&nburnin. NMC=&nmcmc. 
			  THIN=&nthin. JOINTMODEL OUTPOST=out;
	ODS OUTPUT PostSummaries=ds;

	ARRAY data[1] / nosymbols;
	ARRAY a[1] / nosymbols;
	ARRAY b[1] / nosymbols;

	BEGINCNST;
		rc_data = READ_ARRAY("work.tempmcmc",data,"&respname.");
		rc_a    = READ_ARRAY("work.tempmcmc",a,"&aname.");
		rc_b    = READ_ARRAY("work.tempmcmc",b,"&bname.");
		n = dim(data,1);
	ENDCNST;
	 
		PARMS theta 0; *starting value for each theta = 0;
		PRIOR theta~normal(0,var=1); *using 2PL, so assume each theta has standard normal distribution;

		ll=0; *initialize log-likelihood to 0;

		DO i=1 to n;
			prob = 1.701*a[i]*(theta-b[i]);
			prob = exp(prob)/(1+exp(prob));
			if data[i] = 0 then ll = ll+log(1-prob);
			if data[i] = 1 then ll = ll+log(prob);
		END;
		MODEL GENERAL(ll);
	RUN;

	DATA ds; SET ds; internalID = &I;
	DATA thetas; SET thetas DS; RUN;
%END;
ODS LISTING;
PROC PRINTTO; RUN;
DATA thetas; DROP parameter N Mean StdDev p25 p50 p75; 
	SET thetas; IF internalID; thetaestimate = Mean; SDestimate = StdDev; RUN; 

%MEND mcmc_scorePPVT;

%mcmc_scorePPVT(dataname=teststack, orderedID = internalID, respname = resp, aname = mcmc_a, bname = mcmc_b,
				nsubjects = &nsubjects., mcmcseed = &mcmcseed., nburnin = &nburnin., ntune = &ntune., 
				nmcmc = &nmcmc., nthin = &nthin.);

* Give everyone their &personID variable - which is in another data set;
DATA thetas; MERGE thetas responses; BY internalID; 
	theta_est = thetaestimate;
	theta_se = SDestimate;
	KEEP &personID. theta_est theta_se; 
	LABEL &personID.= "&personID.: Original ID variable"
		  theta_est= "theta_est: PPVT Vocabulary Ability Theta Estimate"
		  theta_se=  "theta_se: PPVT Vocabulary Ability	Theta Standard Error";
RUN;

* Output thetas to new file;
DATA filesave.&thetafile.; SET thetas; RUN; 

* Deleting unnecessary files;
PROC DATASETS LIB=WORK NOLIST; 
	DELETE testmult teststack &itemfile. responses Ds Out tempmcmc thetas &respfile.; 
RUN; QUIT;

%MEND PersonScore;
